@@ -1,49 +1,64 @@
 #' Read simple features or layers from file or database
 #'
 #' Read simple features from file or database, or retrieve layer names and their geometry type(s)
-#' @param dsn data source name (interpretation varies by driver - for some drivers, dsn is a file name, but may also be a folder, or contain the name and access credentials of a database)
-#' @param layer layer name (varies by driver, may be a file name without extension); in case \code{layer} is missing, \code{st_read} will read the first layer of \code{dsn}, give a warning and (unless \code{quiet = TRUE}) print a message when there are multiple layers, or give an error if there are no layers in \code{dsn}.
+#' @param dsn data source name (interpretation varies by driver - for some drivers, dsn is a file name, but may also be a folder,
+#' or contain the name and access credentials of a database)
+#' @param layer layer name (varies by driver, may be a file name without extension); in case \code{layer} is missing,
+#' \code{st_read} will read the first layer of \code{dsn}, give a warning and (unless \code{quiet = TRUE}) print a
+#' message when there are multiple layers, or give an error if there are no layers in \code{dsn}.
 #' @param ... parameter(s) passed on to \link{st_as_sf}
 #' @param options character; driver dependent dataset open options, multiple options supported.
 #' @param quiet logical; suppress info on name, driver, size and spatial reference, or signaling no or multiple layers
 #' @param iGeomField integer; in case of multiple geometry fields, which one to take?
-#' @param type integer; ISO number of desired simple feature type; see details. If left zero, in case of mixed feature geometry types, conversion to the highest numeric type value found will be attempted.
-#' @param promote_to_multi logical; in case of a mix of Point and MultiPoint, or of LineString and MultiLineString, or of Polygon and MultiPolygon, convert all to the Multi variety; defaults to \code{TRUE}
-#' @param stringsAsFactors logical; logical: should character vectors be converted to factors?  The `factory-fresh' default is \code{TRUE}, but this can be changed by setting \code{options(stringsAsFactors = FALSE)}.  
-#' @param int64_as_string logical; if TRUE, Int64 attributes are returned as string; if FALSE, they are returned as double and a warning is given when precision is lost (i.e., values are larger than 2^53).
-#' @details for iGeomField, see also \url{https://trac.osgeo.org/gdal/wiki/rfc41_multiple_geometry_fields}; for \code{type} values see \url{https://en.wikipedia.org/wiki/Well-known_text#Well-known_binary}, but note that not every target value may lead to succesful conversion. The typical conversion from POLYGON (3) to MULTIPOLYGON (6) should work; the other way around (type=3), secondary rings from MULTIPOLYGONS may be dropped without warnings. 
-#' @return object of class \link{sf} when a layer was succesfully read; in case argument \code{layer} is missing and data source \code{dsn} does not contain a single layer, an object of class \code{sf_layers} is returned with the layer names, each with their geometry type(s). Note that the number of layers may also be zero.
+#' @param type integer; ISO number of desired simple feature type; see details. If left zero, in case of mixed feature geometry
+#' types, conversion to the highest numeric type value found will be attempted.
+#' @param promote_to_multi logical; in case of a mix of Point and MultiPoint, or of LineString and MultiLineString, or of
+#' Polygon and MultiPolygon, convert all to the Multi variety; defaults to \code{TRUE}
+#' @param stringsAsFactors logical; logical: should character vectors be converted to factors?  The `factory-fresh' default
+#' is \code{TRUE}, but this can be changed by setting \code{options(stringsAsFactors = FALSE)}.
+#' @param int64_as_string logical; if TRUE, Int64 attributes are returned as string; if FALSE, they are returned as double
+#' and a warning is given when precision is lost (i.e., values are larger than 2^53).
+#' @details for iGeomField, see also \url{https://trac.osgeo.org/gdal/wiki/rfc41_multiple_geometry_fields}; for \code{type}
+#' values see \url{https://en.wikipedia.org/wiki/Well-known_text#Well-known_binary}, but note that not every target value
+#' may lead to succesful conversion. The typical conversion from POLYGON (3) to MULTIPOLYGON (6) should work; the other
+#' way around (type=3), secondary rings from MULTIPOLYGONS may be dropped without warnings.
+#' @return object of class \link{sf} when a layer was succesfully read; in case argument \code{layer} is missing and
+#' data source \code{dsn} does not contain a single layer, an object of class \code{sf_layers} is returned with the
+#' layer names, each with their geometry type(s). Note that the number of layers may also be zero.
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
 #' summary(nc)
-#' 
-#' \dontrun{  
+#'
+#' \dontrun{
 #' library(sp)
 #' example(meuse, ask = FALSE, echo = FALSE)
-#' st_write(st_as_sf(meuse), "PG:dbname=postgis", "meuse", 
+#' st_write(st_as_sf(meuse), "PG:dbname=postgis", "meuse",
 #'      layer_options = "OVERWRITE=true")
 #' st_meuse = st_read("PG:dbname=postgis", "meuse")
 #' summary(st_meuse)}
 
 #' @name st_read
-#' @note The use of \code{system.file} in examples make sure that examples run regardless where R is installed: typical users will not use \code{system.file} but give the file name directly, either with full path or relative to the current working directory (see \link{getwd}). "Shapefiles" consist of several files with the same basename that reside in the same directory, only one of them having extension \code{.shp}. 
+#' @note The use of \code{system.file} in examples make sure that examples run regardless where R is installed:
+#' typical users will not use \code{system.file} but give the file name directly, either with full path or relative
+#' to the current working directory (see \link{getwd}). "Shapefiles" consist of several files with the same basename
+#' that reside in the same directory, only one of them having extension \code{.shp}.
 #' @export
 st_read = function(dsn, layer, ..., options = NULL, quiet = FALSE, iGeomField = 1L, type = 0,
-		promote_to_multi = TRUE, stringsAsFactors = default.stringsAsFactors(), 
+		promote_to_multi = TRUE, stringsAsFactors = default.stringsAsFactors(),
 		int64_as_string = FALSE) {
 
 	if (missing(dsn))
 		stop("dsn should specify a data source or filename")
-  
+
 	if (missing(layer))
 		layer = character(0)
-	
+
 	if (file.exists(dsn))
 		dsn = normalizePath(dsn)
 
-	x = CPL_read_ogr(dsn, layer, as.character(options), quiet, iGeomField - 1L, type, 
+	x = CPL_read_ogr(dsn, layer, as.character(options), quiet, iGeomField - 1L, type,
 		promote_to_multi, int64_as_string)
-	which.geom = which(sapply(x, function(f) inherits(f, "sfc")))
+	which.geom = which(vapply(x, function(f) inherits(f, "sfc"), TRUE))
 	nm = names(x)[which.geom]
 	geom = x[[which.geom]]
 	x[[which.geom]] = NULL
@@ -55,36 +70,45 @@ st_read = function(dsn, layer, ..., options = NULL, quiet = FALSE, iGeomField =
 	x = st_as_sf(x, ...)
 	if (! quiet)
 		print(x, n = 0)
-	else 
+	else
 		x
 }
 
 #' @name st_read
 #' @export
-#' @details \code{read_st} and \code{write_sf} are aliases for \code{st_read} and \code{st_write}, respectively.
-read_sf <- function(...) st_read(...)
+#' @details \code{read_sf} and \code{write_sf} are aliases for \code{st_read} and \code{st_write}, respectively, with some
+#' modified default arguments.
+read_sf <- function(..., quiet = TRUE, stringsAsFactors = FALSE)
+	st_read(..., quiet = quiet, stringsAsFactors = stringsAsFactors)
 
 #' @name st_read
 #' @export
-write_sf <- function(...) st_write(...)
+write_sf <- function(..., quiet = TRUE) st_write(..., quiet = quiet)
 
 #' Write simple features object to file or database
 #'
 #' Write simple features object to file or database
 #' @param obj object of class \code{sf} or \code{sfc}
-#' @param dsn data source name (interpretation varies by driver - for some drivers, dsn is a file name, but may also be a folder or contain a database name)
-#' @param layer layer name (varies by driver, may be a file name without extension); if layer is missing, the \link{basename} of \code{dsn} is taken.
-#' @param driver character; driver name to be used, if missing, a driver name is guessed from \code{dsn}; \code{st_drivers()} returns the drivers that are available with their properties; links to full driver documentation are found at \url{http://www.gdal.org/ogr_formats.html}.
+#' @param dsn data source name (interpretation varies by driver - for some drivers, dsn is a file name, but may also be a
+#' folder or contain a database name)
+#' @param layer layer name (varies by driver, may be a file name without extension); if layer is missing, the
+#' \link{basename} of \code{dsn} is taken.
+#' @param driver character; driver name to be used, if missing, a driver name is guessed from \code{dsn};
+#' \code{st_drivers()} returns the drivers that are available with their properties; links to full driver documentation
+#' are found at \url{http://www.gdal.org/ogr_formats.html}.
 #' @param dataset_options character; driver dependent dataset creation options; multiple options supported.
 #' @param layer_options character; driver dependent layer creation options; multiple options supported.
 #' @param quiet logical; suppress info on name, driver, size and spatial reference
-#' @param factorsAsCharacter logical; convert \code{factor} objects into character strings (default), else into numbers by \code{as.numeric}.
+#' @param factorsAsCharacter logical; convert \code{factor} objects into character strings (default), else into numbers by
+#' \code{as.numeric}.
+#' @param update logical; if \code{TRUE}, try to update (append to) existing data source;this is only supported by some drivers
+#' (e.g. GPKG), for other drivers the layer may simply be overwritten
 #' @details columns (variables) of a class not supported are dropped with a warning.
 #' @seealso \link{st_drivers}
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
 #' st_write(nc, "nc.shp")
-#' 
+#'
 #' \dontrun{
 #' library(sp)
 #' example(meuse, ask = FALSE, echo = FALSE)
@@ -93,8 +117,9 @@ write_sf <- function(...) st_write(...)
 #' demo(nc, ask = FALSE)
 #' st_write(nc, "PG:dbname=postgis", "sids", layer_options = "OVERWRITE=true")}
 #' @export
-st_write = function(obj, dsn, layer = basename(dsn), driver = guess_driver_can_write(dsn), ..., 
-		dataset_options = NULL, layer_options = NULL, quiet = FALSE, factorsAsCharacter = TRUE) {
+st_write = function(obj, dsn, layer = basename(dsn), driver = guess_driver_can_write(dsn), ...,
+		dataset_options = NULL, layer_options = NULL, quiet = FALSE, factorsAsCharacter = TRUE,
+		update = FALSE) {
 
 	if (length(list(...)))
 		stop(paste("unrecognized argument(s)", unlist(list(...)), "\n"))
@@ -102,20 +127,20 @@ st_write = function(obj, dsn, layer = basename(dsn), driver = guess_driver_can_w
 	if (inherits(obj, "sfc"))
 		obj = st_sf(id = 1:length(obj), geom = obj)
 	stopifnot(inherits(obj, "sf"))
-	
+
 	if (missing(dsn))
 		stop("dsn should specify a data source or filename")
-	
+
 	if (file.exists(dsn))
 		dsn = normalizePath(dsn)
-	
+
 	geom = st_geometry(obj)
 	obj[[attr(obj, "sf_column")]] = NULL
 	if (factorsAsCharacter)
 		obj = lapply(obj, function(x) if (is.factor(x)) as.character(x) else x)
 	else
 		obj = lapply(obj, function(x) if (is.factor(x)) as.numeric(x) else x)
-	ccls = sapply(obj, function(x) class(x)[1])
+	ccls = vapply(obj, function(x) class(x)[1], "")
 	ccls.ok = ccls %in% c("character", "integer", "numeric", "Date", "POSIXct")
 	if (any(!ccls.ok)) {
 		# nocov start
@@ -124,18 +149,20 @@ st_write = function(obj, dsn, layer = basename(dsn), driver = guess_driver_can_w
 		obj = obj[ccls.ok]
 		# nocov end
 	}
-	attr(obj, "colclasses") = sapply(obj, function(x) class(x)[1])
+	attr(obj, "colclasses") = vapply(obj, function(x) class(x)[1], "")
 	dim = class(geom[[1]])[1]
-	CPL_write_ogr(obj, dsn, layer, driver, 
-		as.character(dataset_options), as.character(layer_options), 
-		geom, dim, quiet)
+	CPL_write_ogr(obj, dsn, layer, driver,
+		as.character(dataset_options), as.character(layer_options),
+		geom, dim, quiet, update)
 }
 
 #' Get GDAL drivers
-#' 
+#'
 #' Get a list of the available GDAL drivers
 #' @param what character: "vector" or "raster", anything else will return all drivers.
-#' @details The drivers available will depend on the installation of GDAL/OGR, and can vary; the \code{st_drivers()} function shows which are available, and which may be written (but all are assumed to be readable). Note that stray files in data source directories (such as *.dbf) may lead to suprious errors that accompanying *.shp are missing.
+#' @details The drivers available will depend on the installation of GDAL/OGR, and can vary; the \code{st_drivers()}
+#' function shows which are available, and which may be written (but all are assumed to be readable). Note that stray
+#' files in data source directories (such as *.dbf) may lead to suprious errors that accompanying *.shp are missing.
 #' @return a \code{data.frame} with driver metadata
 #' @export
 #' @examples
@@ -154,8 +181,8 @@ st_drivers = function(what = "vector") {
 
 #' @export
 print.sf_layers = function(x, ...) {
-	x$geomtype = sapply(x$geomtype, function(x) paste(x, collapse = ", "))
-	cat(paste("Driver:", x$driver, "\n")) 
+	x$geomtype = vapply(x$geomtype, function(x) paste(x, collapse = ", "), "")
+	cat(paste("Driver:", x$driver, "\n"))
 	x$driver = NULL
 	x$features[x$features < 0] = NA
 	cat("Available layers:\n")
@@ -173,7 +200,8 @@ print.sf_layers = function(x, ...) {
 #' List layers in a datasource
 #'
 #' List layers in a datasource
-#' @param dsn data source name (interpretation varies by driver - for some drivers, dsn is a file name, but may also be a folder, or contain the name and access credentials of a database)
+#' @param dsn data source name (interpretation varies by driver - for some drivers, dsn is a file name, but may also be a
+#' folder, or contain the name and access credentials of a database)
 #' @param options character; driver dependent dataset open options, multiple options supported.
 #' @param do_count logical; if TRUE, count the features by reading them, even if their count is not reported by the driver
 #' @name st_layers
@@ -185,23 +213,18 @@ st_layers = function(dsn, options = character(0), do_count = FALSE) {
 		dsn = normalizePath(dsn)
 	CPL_get_layers(dsn, options, do_count)
 }
-#' @name st_layers
-#' @export
-st_list = function(dsn, options = character(0), do_count = FALSE) {
-	.Deprecated("st_layers") # nocov
-}
 
 guess_driver = function(dsn) {
   stopifnot(is.character(dsn))
   stopifnot(length(dsn) == 1)
-  
+
 	# find match: try extension first
 	drv = extension_map[tolower(tools::file_ext(dsn))]
 	if (any(grep(":", gsub(":[/\\]", "/", dsn)))) {
 			drv = prefix_map[tolower(strsplit(dsn, ":")[[1]][1])]
 	}
 	drv <- unlist(drv)
-	
+
 	if (is.null(drv)) {
 	  # no match
 	  return(NA)
@@ -223,7 +246,7 @@ guess_driver_can_write = function(dns, drv = guess_driver(dns)) {
 }
 
 #' Check if driver is available
-#' 
+#'
 #' Search through the driver table if driver is listed
 #' @param drv character. Name of driver
 #' @param drivers data.frame. Table containing driver names and support. Default
@@ -232,13 +255,13 @@ is_driver_available = function(drv, drivers = st_drivers()) {
   i = match(drv, drivers$name)
   if (is.na(i))
     return(FALSE)
-  
+
   return(TRUE)
 }
 
 #' Check if a driver can perform an action
-#' 
-#' Search through the driver table to match a driver name with 
+#'
+#' Search through the driver table to match a driver name with
 #' an action (e.g. \code{"write"}) and check if the action is supported.
 #' @param drv character. Name of driver
 #' @param drivers data.frame. Table containing driver names and support. Default
@@ -249,7 +272,7 @@ is_driver_can = function(drv, drivers = st_drivers(), operation = "write") {
   i = match(drv, drivers$name)
   if (!drivers[i, operation])
     return(FALSE)
-  
+
   return(TRUE)
 }
 
@@ -265,7 +288,7 @@ extension_map <- list(
         "gmt" = "GMT",
         "gpkg" = "GPKG",
         "gps" = "GPSBabel",
-        "gtm" = "GPSTrackMaker",   
+        "gtm" = "GPSTrackMaker",
         "gxt" = "Geoconcept",
         "jml" = "JML",
         "map" = "WAsP",
@@ -292,4 +315,4 @@ prefix_map <- list(
         "oci" = "OCI",
         "odbc" = "ODBC",
         "pg" = "PostgreSQL",
-        "sde" = "SDE") 
+        "sde" = "SDE")
