@@ -4,8 +4,15 @@ nc = st_read(system.file("shape/nc.shp", package="sf"), quiet = TRUE)
 nc %>% filter(AREA > .1) %>% plot()
 
 # plot 10 smallest counties in grey:
-nc %>% plot()
-nc %>% arrange(AREA) %>% slice(1:10) %>% plot(add = TRUE, col = 'grey')
+nc %>% 
+  select(BIR74, geometry) %>% 
+  plot()
+
+nc %>% 
+  select(AREA, geometry) %>% 
+  arrange(AREA) %>% 
+  slice(1:10) %>% 
+  plot(add = TRUE, col = 'grey', main ="")
 
 # select: check both when geometry is part of the selection, and when not:
 nc %>% select(SID74, SID79) %>% names()
@@ -13,9 +20,6 @@ nc %>% select(SID74, SID79, geometry) %>% names()
 nc %>% select(SID74, SID79) %>% class()
 nc %>% select(SID74, SID79, geometry) %>% class()
 
-# arrange: ten smallest counties
-nc %>% arrange(AREA) %>% slice(1:10) %>% plot(add = TRUE, col = 'grey')
-
 # group_by:
 nc$area_cl = cut(nc$AREA, c(0, .1, .12, .15, .25))
 nc %>% group_by(area_cl) %>% class()
@@ -48,25 +52,34 @@ nc %>% select(SID74, SID79, geometry) %>% gather(VAR, SID, -geometry) %>% summar
 
 # spread:
 nc$row = 1:100
-nc.g <- nc %>% select(SID74, SID79, row) %>% gather(VAR, SID, -row)
+nc.g <- nc %>% select(SID74, SID79, row) %>% gather(VAR, SID, -row, -geometry)
 nc.g %>% tail()
 nc.g %>% spread(VAR, SID) %>% head()
 nc %>% select(SID74, SID79, geometry, row) %>% gather(VAR, SID, -geometry, -row) %>% spread(VAR, SID) %>% head()
 
-
-library(dplyr)
-library(sf)
-demo(nc, ask = FALSE, echo = FALSE)
-nc.merc <- st_transform(nc, 3857) # web mercator - not so good!
-nc.merc <- nc.merc %>% mutate(area = st_area(nc.merc), dens = BIR74/area)
-summary(nc.merc$dens)
-nc.merc$area_cl <- cut(nc$AREA, c(0, .1, .12, .15, .25))
-nc.grp <- nc.merc %>% group_by(area_cl)
-out <- nc.grp %>% summarise(A = sum(area), pop = sum(dens * area), new_dens = pop/A) 
-out %>% summarise(sum(A * new_dens))
-nc.merc %>% summarise(sum(area * dens))
-
+# test st_set_crs in pipe:
 sfc = st_sfc(st_point(c(0,0)), st_point(c(1,1)))
 x <- sfc %>% st_set_crs(4326) %>% st_transform(3857)
 x
 
+library(units)
+person = make_unit("person")
+nc = st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
+nc.merc <- st_transform(nc, 32119) # NC State Plane
+
+nc.merc <- nc.merc %>% mutate(area = st_area(nc.merc), dens = BIR74 * person /area)
+
+# summary(nc.merc$dens) # requires units 0.4-2
+nc.merc$area_cl <- cut(nc$AREA, c(0, .1, .12, .15, .25))
+nc.grp <- nc.merc %>% group_by(area_cl)
+
+out <- nc.grp %>% summarise(A = sum(area), pop = sum(dens * area), 
+	new_dens = sum(dens * area)/sum(area)) 
+
+# mean densities depend on grouping:
+nc.merc %>% summarize(mean(dens))
+out %>% summarise(mean(new_dens))
+
+# total densities don't:
+nc.merc %>% summarise(sum(area * dens))
+out %>% summarise(sum(A * new_dens))
