@@ -34,11 +34,6 @@ st_sfc(x, crs = "+proj=longlat +datum=WGS84 +no_defs")
 # but do when it changes:
 st_sfc(x, crs = 3857)
 
-# rbind:
-x = st_sf(a = 1:2, geom = st_sfc(list(st_point(0:1), st_point(0:1)), crs = 4326))
-rbind(x, x, x)
-
-
 p = st_point(0:1)
 st_cast(p, "MULTIPOINT")
 mp = st_multipoint(rbind(c(0,1), c(2,2)))
@@ -95,7 +90,7 @@ x = st_sfc(st_point(c(-90,35)), st_point(c(-80,36)),
 	crs = "+proj=longlat +datum=NAD27")
 st_transform(x, 3857)
 
-sf_extSoftVersion()
+sf_extSoftVersion()[1:3]
 
 # Ops.sfc:
 ls = st_sfc(st_linestring(rbind(c(0,0),c(0,1))))
@@ -145,3 +140,31 @@ st_join(a, b)
 st_join(a, b, left = FALSE)
 st_join(a, b, FUN = mean)
 st_join(a, b, FUN = mean, left = FALSE)
+
+# rbind:
+x = st_sf(a = 1:2, geom = st_sfc(list(st_point(0:1), st_point(0:1)), crs = 4326))
+rbind(x, x, x)
+nc2 = rbind(nc[1:50, ], nc[51:100, ])
+all.equal(nc, nc2)
+
+# st_sample:
+set.seed(131)
+options(digits=6)
+x = st_sfc(st_polygon(list(rbind(c(0,0),c(90,0),c(90,90),c(0,90),c(0,0)))), crs = st_crs(4326))
+(p <- st_sample(x, 10))
+x = st_sfc(st_polygon(list(rbind(c(0,0),c(90,0),c(90,90),c(0,90),c(0,0))))) # NOT long/lat:
+st_sample(x, 10)
+x = st_sfc(st_polygon(list(rbind(c(-180,-90),c(180,-90),c(180,90),c(-180,90),c(-180,-90)))),
+ crs=st_crs(4326))
+p <- st_sample(x, 10)
+pt = st_multipoint(matrix(1:20,,2))
+st_sample(p, 3)
+ls = st_sfc(st_linestring(rbind(c(0,0),c(0,1))),
+ st_linestring(rbind(c(0,0),c(.1,0))),
+ st_linestring(rbind(c(0,1),c(.1,1))),
+ st_linestring(rbind(c(2,2),c(2,2.00001))))
+st_sample(ls, 80)
+st_sample(nc[1:2,], size = c(10,20))
+
+class(st_bind_cols(nc, as.data.frame(nc)[1:3]))
+class(rbind(nc, nc))
